/**	\file 
	<b>USB receive and transmit</b><br>
	Autor: Matthias Weier<br>
	Copyright 2004: Matthias Weier<br>
	License: QPL (see license.txt)
	<hr>
*/

#include <avr/io.h>
#include "tools.h"
#include "usb.h"

//Hardware
#define	USB_TXE		(PINB&BIT6_POS)
#define	USB_RXF		(PINC&BIT3_POS)
#define	USB_RD_ON	PORTC&=BIT1_NEG
#define	USB_RD_OFF	PORTC|=BIT1_POS
#define	USB_WR_ON	PORTC|=BIT2_POS
#define	USB_WR_OFF	PORTC&=BIT2_NEG

/**
	Reads a byte from the USB interface
	
	@return received character or -1 if no character has been received
*/
signed int usb_getc(void)
{
	unsigned char t;
	
	if(!USB_RXF)
	{	
		USB_RD_ON;
		asm("NOP");
		t = PIND;
		USB_RD_OFF;
		
		return t;
	}
	else return -1;
}

/**
	Writes a byte to the USB
	
	@param t Byte to be written	
*/
void usb_putc(unsigned char t)
{
	while(USB_TXE);
	
	//PORTD auf Ausgang
	DDRD = 0xFF;
	//Daten auf den Port
	PORTD = t;
	
	USB_WR_ON;
	USB_WR_OFF;
	
	//PORTC auf Eingang
	DDRD = 0x00;
	//Pull-Ups ein
	PORTD = 0xFF;
}

/**
	Writes a string to USB
	
	@param s The string to be send
*/
void usb_print(unsigned char *s)
{
	while(*s)
	{
		usb_putc(*s);
		s++;
	}
}
